module mdyLcdDispRect( 
    clk_50m     ,  
    rst_n       , 

    lcd_hsync   ,
    lcd_vsync   ,
    lcd_de      ,
 

    lcd_rgb     ,
    lcd_dclk    
       
);

    input               clk_50m     ;  
    input               rst_n       ; 
    output              lcd_hsync   ;
    output              lcd_vsync   ;
    output              lcd_de      ;

    output  [23:0]      lcd_rgb     ;
    output              lcd_dclk    ;
      


    reg                 lcd_hsync   ;
    reg                 lcd_vsync   ;
 
    reg     [23:0]      lcd_rgb     ;


    parameter     LINE_PR   =  1056 ;       
    parameter     FRAME_PER =   525 ;           


    parameter     H_SYNC    =    20 ;       
    parameter     V_SYNC    =    10 ;              

    parameter     HDE_START =    46 ;
    parameter     HDE_END   =   846 ;
    parameter     VDE_START =    23 ;
    parameter     VDE_END   =   503 ;

    

    reg    [12:0]     cnt_hsy       ;
    reg    [12:0]     cnt_vsy       ;
    reg               hsync_de      ;
    reg               vsync_de      ;

    wire              display_area  ;
    wire              e_area        ;
    wire              add_cnt_hsy   ;
    wire              end_cnt_hsy   ;
    wire              add_cnt_vsy   ;
    wire              end_cnt_vsy   ;
    reg [ 7:0]        cnt0          ;
    wire              add_cnt0      ;
    wire              end_cnt0      ;
    reg  [15:0]       cnt1          ;
    wire              add_cnt1      ;
    wire              end_cnt1      ;


    assign clk       = clk_50m              ;
    assign lcd_dclk  = ~ clk_50m            ;
    assign lcd_de    = hsync_de & vsync_de  ;
    
    


    always @ (posedge clk or negedge rst_n)begin
            if(!rst_n)begin
                cnt_hsy <= 0;
            end
            else if(add_cnt_hsy)begin
                if(end_cnt_hsy)
                    cnt_hsy <= 0;
                else
                    cnt_hsy <= cnt_hsy + 1;
            end
        end
    
    assign add_cnt_hsy = 1;
    assign end_cnt_hsy = add_cnt_hsy && cnt_hsy == LINE_PR -1;
    

    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            cnt_vsy <= 0;
        end
        else if(add_cnt_vsy)begin
            if(end_cnt_vsy)
                cnt_vsy <= 0;
            else
                cnt_vsy <= cnt_vsy + 1;
        end
    end
   
    assign add_cnt_vsy = end_cnt_hsy;
    assign end_cnt_vsy = add_cnt_vsy && cnt_vsy == FRAME_PER - 1;
	 

    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            lcd_hsync <= 1'b0 ;
        end
        else if(end_cnt_hsy)begin
            lcd_hsync <= 1'b0;
        end
        else if(add_cnt_hsy && cnt_hsy == H_SYNC-1 )begin
            lcd_hsync <= 1'b1;
        end
    end

    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            hsync_de <= 1'b0;
        end
        else if(add_cnt_hsy && cnt_hsy == HDE_START-1)begin
            hsync_de <= 1'b1;
        end
        else if(add_cnt_hsy && cnt_hsy == HDE_END-1)begin
            hsync_de <= 1'b0;
        end
    end

	 
    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            lcd_vsync <= 1'b0 ;
        end
		  else if(add_cnt_vsy && cnt_vsy == V_SYNC-1 )begin
            lcd_vsync <= 1'b1;
        end
        else if(end_cnt_vsy)begin
            lcd_vsync <= 1'b0;
        end

    end

	 
    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
            vsync_de <= 1'b0;
        end
        else if(add_cnt_vsy && cnt_vsy == VDE_START-1)begin
            vsync_de <= 1'b1;
        end
        else if(add_cnt_vsy && cnt_vsy ==VDE_END-1)begin
            vsync_de <= 1'b0;
        end
    end


    assign   display_area = hsync_de && vsync_de;


    assign   blue_area = (cnt_hsy >= HDE_START + 400-125) && (cnt_hsy<HDE_START+400+125) && 
                         (cnt_vsy >= VDE_START + 240-80) && (cnt_vsy<VDE_START+240+80) ; 

                     
    assign   reb_area  = (cnt_hsy >= HDE_START + 400-250) && (cnt_hsy<HDE_START+400+250) && 
                         (cnt_vsy >= VDE_START + 240-160) && (cnt_vsy<VDE_START+240+160) ; 


    always @ (posedge clk or negedge rst_n)begin
        if(!rst_n)begin
              lcd_rgb <= 0;
        end
        else if(display_area)begin
            if(blue_area)begin
                lcd_rgb <= 24'h00_00_ff ;
            end
            else if(reb_area)begin
                lcd_rgb <= 24'hff_00_00 ;
            end
            else begin
                lcd_rgb <= 24'h00_ff_00 ;
            end
        end
        else begin
            lcd_rgb <= 0;
        end
    end



endmodule

